@@ -293,7 +293,8 @@ module Asciidoctor
   DELIMITED_BLOCK_HEADS = {}.tap {|accum| DELIMITED_BLOCKS.each_key {|k| accum[k.slice 0, 2] = true } }
   DELIMITED_BLOCK_TAILS = {}.tap {|accum| DELIMITED_BLOCKS.each_key {|k| accum[k] = k[k.length - 1] if k.length == 4 } }
 
-  CAPTIONABLE_BLOCKS = { example: true, listing: true, table: true }
+  # NOTE the 'figure' key as a string is historical and used by image blocks
+  CAPTION_ATTR_NAMES = { example: 'example-caption', 'figure' => 'figure-caption', listing: 'listing-caption', table: 'table-caption' }
 
   LAYOUT_BREAK_CHARS = {
     '\'' => :thematic_break,
@@ -1501,7 +1502,7 @@ module Asciidoctor
     end
   end
 
-  # Alias render to convert to maintain backwards compatibility
+  # Deprecated: Use {Asciidoctor.convert} instead.
   alias render convert
 
   # Public: Parse the contents of the AsciiDoc source file into an
@@ -1518,7 +1519,7 @@ module Asciidoctor
     ::File.open(filename, FILE_READ_MODE) {|file| self.convert file, options }
   end
 
-  # Alias render_file to convert_file to maintain backwards compatibility
+  # Deprecated: Use {Asciidoctor.convert_file} instead.
   alias render_file convert_file
 
   # Internal: Automatically load the Asciidoctor::Extensions module.
