@@ -32,9 +32,6 @@ class AbstractBlock < AbstractNode
   # Public: Substitutions to be applied to content in this block.
   attr_reader :subs
 
-  # Internal: Set the default subs applied to this block; used during block creation
-  attr_writer :default_subs
-
   def initialize parent, context, opts = {}
     super
     @content_model = :compound
@@ -77,7 +74,7 @@ class AbstractBlock < AbstractNode
     converter.convert self
   end
 
-  # Alias render to convert to maintain backwards compatibility
+  # Deprecated: Use {AbstractBlock#convert} instead.
   alias render convert
 
   # Public: Get the converted result of the child blocks by converting the
@@ -126,7 +123,7 @@ class AbstractBlock < AbstractNode
   #
   # Returns A Boolean indicating whether this Block has block content
   def blocks?
-    !@blocks.empty?
+    @blocks.empty? ? false : true
   end
 
   # Public: Check whether this block has any child Section objects.
@@ -342,13 +339,13 @@ class AbstractBlock < AbstractNode
       case xrefstyle
       when 'full'
         quoted_title = sub_placeholder (sub_quotes @document.compat_mode ? %q(``%s'') : '"`%s`"'), title
-        if @numeral && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
+        if @numeral && (caption_attr_name = CAPTION_ATTR_NAMES[@context]) && (prefix = @document.attributes[caption_attr_name])
           %(#{prefix} #{@numeral}, #{quoted_title})
         else
           %(#{@caption.chomp '. '}, #{quoted_title})
         end
       when 'short'
-        if @numeral && (prefix = @document.attributes[@context == :image ? 'figure-caption' : %(#{@context}-caption)])
+        if @numeral && (caption_attr_name = CAPTION_ATTR_NAMES[@context]) && (prefix = @document.attributes[caption_attr_name])
           %(#{prefix} #{@numeral})
         else
           @caption.chomp '. '
@@ -372,16 +369,16 @@ class AbstractBlock < AbstractNode
   # The parts of a complete caption are: <prefix> <number>. <title>
   # This partial caption represents the part the precedes the title.
   #
-  # value - The explicit String caption to assign to this block (default: nil).
-  # key   - The String prefix for the caption and counter attribute names.
-  #         If not provided, the name of the context for this block is used.
-  #         (default: @context)
+  # value           - The explicit String caption to assign to this block (default: nil).
+  # caption_context - The Symbol context to use when resolving caption-related attributes.
+  #                   If not provided, the name of the context for this block is used.
+  #                   (default: @context)
   #
   # Returns nothing.
-  def assign_caption value = nil, key = @context
+  def assign_caption value = nil, caption_context = @context
     unless @caption || !@title || (@caption = value || @document.attributes['caption'])
-      if (prefix = @document.attributes[%(#{key}-caption)])
-        @caption = %(#{prefix} #{@numeral = @document.increment_and_store_counter %[#{key}-number], self}. )
+      if (attr_name = CAPTION_ATTR_NAMES[caption_context]) && (prefix = @document.attributes[attr_name])
+        @caption = %(#{prefix} #{@numeral = @document.increment_and_store_counter %(#{caption_context}-number), self}. )
         nil
       end
     end
@@ -435,8 +432,10 @@ class AbstractBlock < AbstractNode
     end
   end
 
+  protected
+
   # Internal: Performs the work for find_by, but does not handle the StopIteration exception.
-  protected def find_by_internal selector = {}, result = [], &block
+  def find_by_internal selector = {}, result = [], &block
     if ((any_context = (context_selector = selector[:context]) ? nil : true) || context_selector == @context) &&
         (!(style_selector = selector[:style]) || style_selector == @style) &&
         (!(role_selector = selector[:role]) || (has_role? role_selector)) &&
