@@ -956,7 +956,7 @@ class Document < AbstractBlock
     output
   end
 
-  # Alias render to convert to maintain backwards compatibility
+  # Deprecated: Use {Document#convert} instead.
   alias render convert
 
   # Public: Write the output to the specified file
@@ -1102,7 +1102,8 @@ class Document < AbstractBlock
   # Returns The String value with substitutions performed
   def apply_attribute_value_subs value
     if AttributeEntryPassMacroRx =~ value
-      value = $1 ? (apply_subs $2, (resolve_pass_subs $1)) : $2
+      value = $2
+      value = apply_subs value, (resolve_pass_subs $1) if $1
     else
       value = apply_header_subs value
     end
@@ -1326,7 +1327,7 @@ class Document < AbstractBlock
           attrs['outfilesuffix'] = converter.outfilesuffix unless attribute_locked? 'outfilesuffix'
         end
       elsif converter
-        backend_traits = Converter::BackendTraits.derive_backend_traits new_backend
+        backend_traits = Converter.derive_backend_traits new_backend
         new_basebackend = backend_traits[:basebackend]
         new_filetype = backend_traits[:filetype]
         if init
